from datetime import datetime
from typing import List, Optional, Dict, Any

from janis_core import *
from janis_core.types.common_data_types import String, File, Boolean, Int

Blasr_V0_1_0 = CommandToolBuilder(tool="blasr", base_command=["blasr"], inputs=[ToolInput(tag="in_ctab", input_type=String(optional=True), prefix="-ctab", doc=InputDocumentation(doc="A table of tuple counts used to estimate match significance.  This is\nby the program 'printTupleCountTable'.  While it is quick to generate on\nthe fly, if there are many invocations of blasr, it is useful to\nprecompute the ctab.")), ToolInput(tag="in_region_table", input_type=File(optional=True), prefix="-regionTable", doc=InputDocumentation(doc="Read in a read-region table in HDF format for masking portions of reads.\nThis may be a single table if there is just one input file,\nor a fofn.  When a region table is specified, any region table inside\nthe reads.plx.h5 or reads.bax.h5 files are ignored.")), ToolInput(tag="in_use_ccs", input_type=Boolean(optional=True), prefix="-useccs", doc=InputDocumentation(doc="Align the circular consensus sequence (ccs), then report alignments\nof the ccs subreads to the window that the ccs was mapped to.  Only\nalignments of the subreads are reported.")), ToolInput(tag="in_use_ccs_all", input_type=Boolean(optional=True), prefix="-useccsall", doc=InputDocumentation(doc="Similar to -useccs, except all subreads are aligned, rather than just\nthe subreads used to call the ccs.  This will include reads that only\ncover part of the template.")), ToolInput(tag="in_use_ccs_de_novo", input_type=Boolean(optional=True), prefix="-useccsdenovo", doc=InputDocumentation(doc="Align the circular consensus, and report only the alignment of the ccs\nsequence.")), ToolInput(tag="in_no_split_sub_reads", input_type=Boolean(optional=True), prefix="-noSplitSubreads", doc=InputDocumentation(doc="(false)\nDo not split subreads at adapters.  This is typically only\nuseful when the genome in an unrolled version of a known template, and\ncontains template-adapter-reverse_template sequence.")), ToolInput(tag="in_ignore_regions", input_type=Boolean(optional=True), prefix="-ignoreRegions", doc=InputDocumentation(doc="(false)\nIgnore any information in the region table.")), ToolInput(tag="in_ignore_hq_regions", input_type=Boolean(optional=True), prefix="-ignoreHQRegions", doc=InputDocumentation(doc="(false)Ignore any hq regions in the region table.")), ToolInput(tag="in_best_n", input_type=Int(optional=True), prefix="-bestn", doc=InputDocumentation(doc="(10)\nReport the top 'n' alignments.")), ToolInput(tag="in_sam", input_type=Boolean(optional=True), prefix="-sam", doc=InputDocumentation(doc="Write output in SAM format.")), ToolInput(tag="in_clipping", input_type=Boolean(optional=True), prefix="-clipping", doc=InputDocumentation(doc="[none|hard|subread|soft] (none)\nUse no/hard/subread/soft clipping for SAM output.")), ToolInput(tag="in_out", input_type=String(optional=True), prefix="-out", doc=InputDocumentation(doc="(terminal)\nWrite output to 'out'.")), ToolInput(tag="in_unaligned", input_type=File(optional=True), prefix="-unaligned", doc=InputDocumentation(doc="Output reads that are not aligned to 'file'")), ToolInput(tag="in_printing_sam_modify", input_type=Int(optional=True), prefix="-m", doc=InputDocumentation(doc="If not printing SAM, modify the output of the alignment.\nt=0 Print blast like output with |'s connecting matched nucleotides.\n1 Print only a summary: score and pos.\n2 Print in Compare.xml format.\n3 Print in vulgar format (deprecated).\n4 Print a longer tabular version of the alignment.\n5 Print in a machine-parsable format that is read by compareSequences.py.")), ToolInput(tag="in_no_sort_refined_alignments", input_type=Boolean(optional=True), prefix="-noSortRefinedAlignments", doc=InputDocumentation(doc="(false)\nOnce candidate alignments are generated and scored via sparse dynamic\nprogramming, they are rescored using local alignment that accounts\nfor different error profiles.\nResorting based on the local alignment may change the order the hits are returned.")), ToolInput(tag="in_allow_adjacent_indels", input_type=Boolean(optional=True), prefix="-allowAdjacentIndels", doc=InputDocumentation(doc="When specified, adjacent insertion or deletions are allowed. Otherwise, adjacent\ninsertion and deletions are merged into one operation.  Using quality values\nto guide pairwise alignments may dictate that the higher probability alignment\ncontains adjacent insertions or deletions.  Current tools such as GATK do not permit\nthis and so they are not reported by default.")), ToolInput(tag="in_header", input_type=File(optional=True), prefix="-header", doc=InputDocumentation(doc="Print a header as the first line of the output file describing the contents of each column.")), ToolInput(tag="in_title_table", input_type=Int(optional=True), prefix="-titleTable", doc=InputDocumentation(doc="(NULL)\nConstruct a table of reference sequence titles.  The reference sequences are\nenumerated by row, 0,1,...  The reference index is printed in alignment results\nrather than the full reference name.  This makes output concise, particularly when\nvery verbose titles exist in reference names.")), ToolInput(tag="in_min_pct_identity", input_type=Int(optional=True), prefix="-minPctIdentity", doc=InputDocumentation(doc="(0)\nOnly report alignments if they are greater than p percent identity.")), ToolInput(tag="in_hole_numbers", input_type=String(optional=True), prefix="-holeNumbers", doc=InputDocumentation(doc="When specified, only align reads whose ZMW hole numbers are in LIST.\nLIST is a comma-delimited string of ranges, such as '1,2,3,10-13'.\nThis option only works when reads are in base or pulse h5 format.")), ToolInput(tag="in_place_repeats_randomly", input_type=Boolean(optional=True), prefix="-placeRepeatsRandomly", doc=InputDocumentation(doc="(false)\nWhen there are multiple positions to map a read with equal alignment scores, place the\nread randomly at one of them.")), ToolInput(tag="in_prints_am_qv", input_type=Boolean(optional=True), prefix="-printSAMQV", doc=InputDocumentation(doc="(false)\nPrint quality values to sam files.")), ToolInput(tag="in_min_match", input_type=Int(optional=True), prefix="-minMatch", doc=InputDocumentation(doc="(12)\nMinimum seed length.  Higher minMatch will speed up alignment,\nbut decrease sensitivity.")), ToolInput(tag="in_max_match", input_type=Int(optional=True), prefix="-maxMatch", doc=InputDocumentation(doc="(inf)\nStop mapping a read to the genome when the lcp length reaches l.\nThis is useful when the query is part of the reference, for example when\nconstructing pairwise alignments for de novo assembly.")), ToolInput(tag="in_max_lcp_length", input_type=String(optional=True), prefix="-maxLCPLength", doc=InputDocumentation(doc="(inf)\nThe same as -maxMatch.")), ToolInput(tag="in_max_anchors_per_position", input_type=Int(optional=True), prefix="-maxAnchorsPerPosition", doc=InputDocumentation(doc="(10000)\nDo not add anchors from a position if it matches to more than 'm' locations in the target.")), ToolInput(tag="in_advance_exact_matches", input_type=Int(optional=True), prefix="-advanceExactMatches", doc=InputDocumentation(doc="(0)\nAnother trick for speeding up alignments with match - E fewer anchors.  Rather than\nfinding anchors between the read and the genome at every position in the read,\nwhen an anchor is found at position i in a read of length L, the next position\nin a read to find an anchor is at i+L-E.\nUse this when alignining already assembled contigs.")), ToolInput(tag="in_n_candidates", input_type=Int(optional=True), prefix="-nCandidates", doc=InputDocumentation(doc="(10)\nKeep up to 'n' candidates for the best alignment.  A large value of n will slow mapping\nbecause the slower dynamic programming steps are applied to more clusters of anchors\nwhich can be a rate limiting step when reads are very long.")), ToolInput(tag="in_concordant", input_type=Boolean(optional=True), prefix="-concordant", doc=InputDocumentation(doc="(false)\nMap all subreads of a zmw (hole) to where the longest full pass subread of the zmw\naligned to. This requires to use the region table and hq regions.\nThis option only works when reads are in base or pulse h5 format.")), ToolInput(tag="in_concordant_template", input_type=Boolean(optional=True), prefix="-concordantTemplate", doc=InputDocumentation(doc="(mediansubread)\nSelect a full pass subread of a zmw as template for concordant mapping.\nlongestsubread - use the longest full pass subread\nmediansubread  - use the median length full pass subread\ntypicalsubread - use the second longest full pass subread if length of\nthe longest full pass subread is an outlier")), ToolInput(tag="in_fast_max_interval", input_type=Boolean(optional=True), prefix="-fastMaxInterval", doc=InputDocumentation(doc="(false)\nFast search maximum increasing intervals as alignment candidates. The search\nis not as exhaustive as the default, but is much faster.")), ToolInput(tag="in_aggressive_interval_cut", input_type=Boolean(optional=True), prefix="-aggressiveIntervalCut", doc=InputDocumentation(doc="(false)\nAgreesively filter out non-promising alignment candidates, if there\nexists at least one promising candidate. If this option is turned on,\nBlasr is likely to ignore short alignments of ALU elements.")), ToolInput(tag="in_fast_sdp", input_type=Boolean(optional=True), prefix="-fastSDP", doc=InputDocumentation(doc="(false)\nUse a fast heuristic algorithm to speed up sparse dynamic programming.")), ToolInput(tag="in_sdp_tuple_size", input_type=Int(optional=True), prefix="-sdpTupleSize", doc=InputDocumentation(doc="(11)\nUse matches of length K to speed dynamic programming alignments.  This controls\naccuracy of assigning gaps in pairwise alignments once a mapping has been found,\nrather than mapping sensitivity itself.")), ToolInput(tag="in_score_matrix", input_type=Boolean(optional=True), prefix="-scoreMatrix", doc=InputDocumentation(doc="'score matrix string'\nSpecify an alternative score matrix for scoring fasta reads.  The matrix is\nin the format\nACGTN\nA abcde\nC fghij\nG klmno\nT pqrst\nN uvwxy . The values a...y should be input as a quoted space separated\nstring: 'a b c ... y'. Lower scores are better, so matches should be less\nthan mismatches e.g. a,g,m,s = -5 (match), mismatch = 6.")), ToolInput(tag="in_affine_open", input_type=Int(optional=True), prefix="-affineOpen", doc=InputDocumentation(doc="(10)\nSet the penalty for opening an affine alignment.")), ToolInput(tag="in_affine_extend", input_type=Int(optional=True), prefix="-affineExtend", doc=InputDocumentation(doc="(0)\nChange affine (extension) gap penalty. Lower value allows more gaps.")), ToolInput(tag="in_use_quality", input_type=Boolean(optional=True), prefix="-useQuality", doc=InputDocumentation(doc="(false)\nUse substitution/insertion/deletion/merge quality values to score gap and\nmismatch penalties in pairwise alignments.  Because the insertion and deletion\nrates are much higher than substitution, this will make many alignments\nfavor an insertion/deletion over a substitution.  Naive consensus calling methods\nwill then often miss substitution polymorphisms. This option should be\nused when calling consensus using the Quiver method.  Furthermore, when\nnot using quality values to score alignments, there will be a lower consensus\naccuracy in homolymer regions.")), ToolInput(tag="in_affine_align", input_type=Boolean(optional=True), prefix="-affineAlign", doc=InputDocumentation(doc="(false)\nRefine alignment using affine guided align.")), ToolInput(tag="in_min_read_length", input_type=Int(optional=True), prefix="-minReadLength", doc=InputDocumentation(doc="(50)\nSkip reads that have a full length less than l. Subreads may be shorter.")), ToolInput(tag="in_min_sub_read_length", input_type=Int(optional=True), prefix="-minSubreadLength", doc=InputDocumentation(doc="(0)\nDo not align subreads of length less than l.")), ToolInput(tag="in_max_score", input_type=Int(optional=True), prefix="-maxScore", doc=InputDocumentation(doc="(0)\nMaximum score to output (high is bad, negative good).")), ToolInput(tag="in_nproc", input_type=Int(optional=True), prefix="-nproc", doc=InputDocumentation(doc="(1)\nAlign using N processes.  All large data structures such as the suffix array and\ntuple count table are shared.")), ToolInput(tag="in_start", input_type=Int(optional=True), prefix="-start", doc=InputDocumentation(doc="(0)\nIndex of the first read to begin aligning. This is useful when multiple instances\nare running on the same data, for example when on a multi-rack cluster.")), ToolInput(tag="in_stride", input_type=Int(optional=True), prefix="-stride", doc=InputDocumentation(doc="(1)\nAlign one read every 'S' reads.")), ToolInput(tag="in_subsample", input_type=Boolean(optional=True), prefix="-subsample", doc=InputDocumentation(doc="(0)\nProportion of reads to randomly subsample (expressed as a decimal) and align.")), ToolInput(tag="in_maintained_dot", input_type=String(), position=0, doc=InputDocumentation(doc="The extra quality information improves variant detection and mapping")), ToolInput(tag="in_speed_dot", input_type=String(), position=1, doc=InputDocumentation(doc="-sa suffixArrayFile")), ToolInput(tag="in_reads_dot", input_type=String(), position=0, doc=InputDocumentation(doc="Reads produced by spurrious reads from empty ZMWs have a high"))], outputs=[ToolOutput(tag="out_unaligned", output_type=File(optional=True), selector=InputSelector(input_to_select="in_unaligned", type_hint=File()), doc=OutputDocumentation(doc="Output reads that are not aligned to 'file'")), ToolOutput(tag="out_header", output_type=File(optional=True), selector=InputSelector(input_to_select="in_header", type_hint=File()), doc=OutputDocumentation(doc="Print a header as the first line of the output file describing the contents of each column."))], container=None, version="v0.1.0")


if __name__ == "__main__":
    # or "cwl"
    Blasr_V0_1_0().translate("wdl", allow_empty_container=True)

